#!perl -T
use strict;
use warnings;
use Test::More tests => 1;

# Check if compiles:
BEGIN
{
	require_ok("WWW::Challonge") || BAIL_OUT();
}

diag("Testing WWW::Challonge $WWW::Challonge::VERSION, Perl $], $^X");
